from django.db import models
from wagtail.admin.edit_handlers import (
    FieldPanel,
    ObjectList,
    StreamFieldPanel,
    TabbedInterface,
)
from wagtail.contrib.settings.models import BaseSetting, register_setting
from wagtail.core import blocks
from wagtail.core.fields import RichTextField, StreamField
from wagtail.core.models import Page
from wagtail.documents.blocks import DocumentChooserBlock
from wagtail.images.blocks import ImageChooserBlock
from wagtail.images.edit_handlers import ImageChooserPanel


class SenatCampaignHomePage(Page):
    # top section
    headline = models.CharField("podtitulek pod jménem", max_length=250, blank=True)
    top_photo = models.ForeignKey(
        "wagtailimages.Image",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        verbose_name="hlavní fotka",
    )
    claim = models.CharField("slogan pod fotkou", max_length=250, blank=True, null=True)
    # about
    about_left = RichTextField("kdo jsem (levý sloupec)", blank=True)
    about_right = RichTextField("kdo jsem (pravý sloupec)", blank=True)
    about_gallery = StreamField(
        [("photo", ImageChooserBlock(label="fotka"))],
        verbose_name="kdo jsem - galerie",
        blank=True,
    )
    # financials
    financials = StreamField(
        [
            (
                "link",
                blocks.StructBlock(
                    [
                        ("label", blocks.CharBlock(label="název")),
                        ("url", blocks.URLBlock(label="odkaz")),
                    ],
                    label="odkaz",
                ),
            ),
            (
                "document",
                blocks.StructBlock(
                    [
                        ("label", blocks.CharBlock(label="název")),
                        ("doc", DocumentChooserBlock(label="dokument")),
                    ],
                    label="dokument",
                ),
            ),
        ],
        verbose_name="transparentní financování",
        blank=True,
    )

    content_panels = Page.content_panels + [
        FieldPanel("headline"),
        ImageChooserPanel("top_photo"),
        FieldPanel("claim"),
        FieldPanel("about_left", classname="full"),
        FieldPanel("about_right", classname="full"),
        StreamFieldPanel("about_gallery"),
        StreamFieldPanel("financials"),
    ]

    subpage_types = [
        "senat_campaign.SenatCampaignNewsIndexPage",
        "senat_campaign.SenatCampaignProgramPage",
        "senat_campaign.SenatCampaignCookiesPage",
    ]

    # flag for rendering anchor links in menu
    is_home = True

    class Meta:
        verbose_name = f"Senát kampaň"

    def get_context(self, request):
        context = super().get_context(request)

        # get news
        context["articles"] = (
            self.get_descendants()
            .type(SenatCampaignNewsPage)
            .live()
            .order_by("-senatcampaignnewspage__date")[:3]
        )

        try:
            context["news_url"] = (
                self.get_children()
                .type(SenatCampaignNewsIndexPage)
                .live()
                .get()
                .get_url(request)
            )
        except Page.DoesNotExist:
            context["news_url"] = "#"

        # get page with program
        try:
            context["program_page"] = (
                self.get_children()
                .type(SenatCampaignProgramPage)
                .live()
                .specific()
                .get()
            )
        except Page.DoesNotExist:
            context["program_page"] = None

        return context


class SenatCampaignNewsIndexPage(Page):
    parent_page_types = ["senat_campaign.SenatCampaignHomePage"]
    subpage_types = ["senat_campaign.SenatCampaignNewsPage"]

    # flag for rendering anchor links in menu
    is_home = False

    class Meta:
        verbose_name = f"Aktuality"

    def get_context(self, request):
        context = super().get_context(request)
        articles = self.get_children().live().order_by("-senatcampaignnewspage__date")
        context["articles"] = articles
        return context


class SenatCampaignNewsPage(Page):
    date = models.DateField("datum")
    perex = models.TextField("perex")
    body = RichTextField("článek", blank=True)
    photo = models.ForeignKey(
        "wagtailimages.Image",
        verbose_name="fotka",
        on_delete=models.PROTECT,
        null=True,
        blank=True,
    )

    content_panels = Page.content_panels + [
        FieldPanel("date"),
        FieldPanel("perex"),
        FieldPanel("body", classname="full"),
        ImageChooserPanel("photo"),
    ]

    parent_page_types = ["senat_campaign.SenatCampaignNewsIndexPage"]
    subpage_types = []

    # flag for rendering anchor links in menu
    is_home = False

    class Meta:
        verbose_name = f"Aktualita"

    def get_context(self, request):
        context = super().get_context(request)
        context["related_articles"] = (
            self.get_siblings(inclusive=False)
            .live()
            .order_by("-senatcampaignnewspage__date")[:3]
        )
        return context


class ProgramBlock(blocks.StructBlock):
    title = blocks.CharBlock(label="titulek")
    perex = blocks.TextBlock(label="perex na úvodní stránku")
    body = blocks.RichTextBlock(label="text bodu")
    image = ImageChooserBlock(label="ilustrační obrázek")

    class Meta:
        icon = "doc-full"
        label = "programový bod"


class SenatCampaignProgramPage(Page):
    committee_preference = StreamField(
        [("committee", blocks.CharBlock(label="výbor či komise"))],
        verbose_name="preferované výbory a komise",
        blank=True,
    )
    content = StreamField(
        [("item", ProgramBlock())], verbose_name="programové body", blank=True
    )

    content_panels = Page.content_panels + [
        StreamFieldPanel("committee_preference"),
        StreamFieldPanel("content"),
    ]

    parent_page_types = ["senat_campaign.SenatCampaignHomePage"]
    subpage_types = []

    # flag for rendering anchor links in menu
    is_home = False

    class Meta:
        verbose_name = f"Program"


class SenatCampaignCookiesPage(Page):
    body = RichTextField("obsah", blank=True)

    content_panels = Page.content_panels + [
        FieldPanel("body", classname="full"),
    ]

    parent_page_types = ["senat_campaign.SenatCampaignHomePage"]
    subpage_types = []

    # flag for rendering anchor links in menu
    is_home = False

    class Meta:
        verbose_name = f"Cookies"


class ContactBlock(blocks.StructBlock):
    name = blocks.CharBlock(label="jméno")
    job = blocks.CharBlock(label="pozice", required=False)
    phone = blocks.CharBlock(label="telefon", required=False)
    email = blocks.EmailBlock(label="email", required=False)
    photo = ImageChooserBlock(label="fotka")

    class Meta:
        icon = "person"
        label = "kontaktní osoba"


@register_setting
class SenatCampaignWebSettings(BaseSetting):
    first_name = models.CharField("jméno kandidáta", max_length=250)
    last_name = models.CharField("příjmení kandidáta", max_length=250)
    facebook = models.URLField("Facebook URL", blank=True, null=True)
    instagram = models.URLField("Instagram URL", blank=True, null=True)
    twitter = models.URLField("Twitter URL", blank=True, null=True)
    linkedin = models.URLField("LinkedIn URL", blank=True, null=True)
    contacts = StreamField(
        [("item", ContactBlock())], verbose_name="kontaktní osoby", blank=True
    )

    # TODO Matomo
    # TODO donations

    first_tab_panels = [
        FieldPanel("first_name"),
        FieldPanel("last_name"),
    ]

    second_tab_panels = [
        FieldPanel("facebook"),
        FieldPanel("instagram"),
        FieldPanel("twitter"),
        FieldPanel("linkedin"),
    ]

    third_tab_panels = [
        StreamFieldPanel("contacts"),
    ]

    edit_handler = TabbedInterface(
        [
            ObjectList(first_tab_panels, heading="Nastavení"),
            ObjectList(second_tab_panels, heading="Sociální sítě"),
            ObjectList(third_tab_panels, heading="Kontakty"),
        ]
    )

    class Meta:
        verbose_name = f"Senát kampaň"

    @property
    def full_name(self):
        return f"{self.first_name} {self.last_name}"

    @property
    def has_program(self):
        return SenatCampaignProgramPage.objects.in_site(self.site).live().exists()

    @property
    def has_news(self):
        return SenatCampaignNewsIndexPage.objects.in_site(self.site).live().exists()

    @property
    def has_callendar(self):
        # TODO
        return False

    @property
    def has_donations(self):
        # TODO
        return False

    @property
    def cookies_page_url(self):
        try:
            return (
                SenatCampaignCookiesPage.objects.in_site(self.site)
                .live()
                .get()
                .get_url()
            )
        except Page.DoesNotExist:
            return "#"
